# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Basic Service."""

__version__ = "2.1.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_get_user_profile__eda78b import AdminGetUserProfilePublicInfoByIds
from .create_my_profile import CreateMyProfile
from .delete_user_profile import DeleteUserProfile
from .get_custom_attributes_info import GetCustomAttributesInfo
from .get_my_profile_info import GetMyProfileInfo
from .get_my_zip_code import GetMyZipCode
from .get_private_custom_attr_322032 import GetPrivateCustomAttributesInfo
from .get_user_profile_info import GetUserProfileInfo
from .get_user_profile_info_b_5dfc7f import GetUserProfileInfoByPublicId
from .public_create_user_profile import PublicCreateUserProfile
from .public_get_custom_attri_5323c2 import PublicGetCustomAttributesInfo
from .public_get_user_profile_info import PublicGetUserProfileInfo
from .public_get_user_profile_0d683e import PublicGetUserProfileInfoByPublicId
from .public_get_user_profile_64cc50 import PublicGetUserProfilePublicInfo
from .public_get_user_profile_cdd77a import PublicGetUserProfilePublicInfoByIds
from .public_update_custom_at_7dcf23 import PublicUpdateCustomAttributesPartially
from .public_update_user_profile import PublicUpdateUserProfile
from .public_update_user_prof_c93172 import PublicUpdateUserProfileStatus
from .update_custom_attribute_105f42 import UpdateCustomAttributesPartially
from .update_my_profile import UpdateMyProfile
from .update_my_zip_code import UpdateMyZipCode
from .update_private_custom_a_62b74f import UpdatePrivateCustomAttributesPartially
from .update_user_profile import UpdateUserProfile
from .update_user_profile_status import UpdateUserProfileStatus



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Legal Service (1.22.6)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.localized_policy_version_object import LocalizedPolicyVersionObject


class PolicyVersionWithLocalizedVersionObject(Model):
    """Policy version with localized version object (PolicyVersionWithLocalizedVersionObject)

    Properties:
        display_version: (displayVersion) REQUIRED str

        id_: (id) REQUIRED str

        is_committed: (isCommitted) REQUIRED bool

        is_in_effect: (isInEffect) REQUIRED bool

        created_at: (createdAt) OPTIONAL str

        description: (description) OPTIONAL str

        localized_policy_versions: (localizedPolicyVersions) OPTIONAL List[LocalizedPolicyVersionObject]

        published_date: (publishedDate) OPTIONAL str

        status: (status) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    display_version: str  # REQUIRED
    id_: str  # REQUIRED
    is_committed: bool  # REQUIRED
    is_in_effect: bool  # REQUIRED
    created_at: str  # OPTIONAL
    description: str  # OPTIONAL
    localized_policy_versions: List[LocalizedPolicyVersionObject]  # OPTIONAL
    published_date: str  # OPTIONAL
    status: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_display_version(
        self, value: str
    ) -> PolicyVersionWithLocalizedVersionObject:
        self.display_version = value
        return self

    def with_id(self, value: str) -> PolicyVersionWithLocalizedVersionObject:
        self.id_ = value
        return self

    def with_is_committed(self, value: bool) -> PolicyVersionWithLocalizedVersionObject:
        self.is_committed = value
        return self

    def with_is_in_effect(self, value: bool) -> PolicyVersionWithLocalizedVersionObject:
        self.is_in_effect = value
        return self

    def with_created_at(self, value: str) -> PolicyVersionWithLocalizedVersionObject:
        self.created_at = value
        return self

    def with_description(self, value: str) -> PolicyVersionWithLocalizedVersionObject:
        self.description = value
        return self

    def with_localized_policy_versions(
        self, value: List[LocalizedPolicyVersionObject]
    ) -> PolicyVersionWithLocalizedVersionObject:
        self.localized_policy_versions = value
        return self

    def with_published_date(
        self, value: str
    ) -> PolicyVersionWithLocalizedVersionObject:
        self.published_date = value
        return self

    def with_status(self, value: str) -> PolicyVersionWithLocalizedVersionObject:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> PolicyVersionWithLocalizedVersionObject:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "display_version"):
            result["displayVersion"] = str(self.display_version)
        elif include_empty:
            result["displayVersion"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "is_committed"):
            result["isCommitted"] = bool(self.is_committed)
        elif include_empty:
            result["isCommitted"] = False
        if hasattr(self, "is_in_effect"):
            result["isInEffect"] = bool(self.is_in_effect)
        elif include_empty:
            result["isInEffect"] = False
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "localized_policy_versions"):
            result["localizedPolicyVersions"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.localized_policy_versions
            ]
        elif include_empty:
            result["localizedPolicyVersions"] = []
        if hasattr(self, "published_date"):
            result["publishedDate"] = str(self.published_date)
        elif include_empty:
            result["publishedDate"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        display_version: str,
        id_: str,
        is_committed: bool,
        is_in_effect: bool,
        created_at: Optional[str] = None,
        description: Optional[str] = None,
        localized_policy_versions: Optional[List[LocalizedPolicyVersionObject]] = None,
        published_date: Optional[str] = None,
        status: Optional[str] = None,
        updated_at: Optional[str] = None,
    ) -> PolicyVersionWithLocalizedVersionObject:
        instance = cls()
        instance.display_version = display_version
        instance.id_ = id_
        instance.is_committed = is_committed
        instance.is_in_effect = is_in_effect
        if created_at is not None:
            instance.created_at = created_at
        if description is not None:
            instance.description = description
        if localized_policy_versions is not None:
            instance.localized_policy_versions = localized_policy_versions
        if published_date is not None:
            instance.published_date = published_date
        if status is not None:
            instance.status = status
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PolicyVersionWithLocalizedVersionObject:
        instance = cls()
        if not dict_:
            return instance
        if "displayVersion" in dict_ and dict_["displayVersion"] is not None:
            instance.display_version = str(dict_["displayVersion"])
        elif include_empty:
            instance.display_version = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "isCommitted" in dict_ and dict_["isCommitted"] is not None:
            instance.is_committed = bool(dict_["isCommitted"])
        elif include_empty:
            instance.is_committed = False
        if "isInEffect" in dict_ and dict_["isInEffect"] is not None:
            instance.is_in_effect = bool(dict_["isInEffect"])
        elif include_empty:
            instance.is_in_effect = False
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if (
            "localizedPolicyVersions" in dict_
            and dict_["localizedPolicyVersions"] is not None
        ):
            instance.localized_policy_versions = [
                LocalizedPolicyVersionObject.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["localizedPolicyVersions"]
            ]
        elif include_empty:
            instance.localized_policy_versions = []
        if "publishedDate" in dict_ and dict_["publishedDate"] is not None:
            instance.published_date = str(dict_["publishedDate"])
        elif include_empty:
            instance.published_date = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PolicyVersionWithLocalizedVersionObject]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PolicyVersionWithLocalizedVersionObject]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PolicyVersionWithLocalizedVersionObject,
        List[PolicyVersionWithLocalizedVersionObject],
        Dict[Any, PolicyVersionWithLocalizedVersionObject],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "displayVersion": "display_version",
            "id": "id_",
            "isCommitted": "is_committed",
            "isInEffect": "is_in_effect",
            "createdAt": "created_at",
            "description": "description",
            "localizedPolicyVersions": "localized_policy_versions",
            "publishedDate": "published_date",
            "status": "status",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "displayVersion": True,
            "id": True,
            "isCommitted": True,
            "isInEffect": True,
            "createdAt": False,
            "description": False,
            "localizedPolicyVersions": False,
            "publishedDate": False,
            "status": False,
            "updatedAt": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.14.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum


class StatusEnum(StrEnum):
    ACQUIRED = "ACQUIRED"
    ACTIVE = "ACTIVE"


class KeyInfo(Model):
    """Key info (KeyInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        key_file: (keyFile) REQUIRED str

        key_group_id: (keyGroupId) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        updated_at: (updatedAt) REQUIRED str

        value: (value) REQUIRED str

        acquire_order_no: (acquireOrderNo) OPTIONAL str

        acquire_user_id: (acquireUserId) OPTIONAL str
    """

    # region fields

    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    key_file: str  # REQUIRED
    key_group_id: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    updated_at: str  # REQUIRED
    value: str  # REQUIRED
    acquire_order_no: str  # OPTIONAL
    acquire_user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> KeyInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> KeyInfo:
        self.id_ = value
        return self

    def with_key_file(self, value: str) -> KeyInfo:
        self.key_file = value
        return self

    def with_key_group_id(self, value: str) -> KeyInfo:
        self.key_group_id = value
        return self

    def with_namespace(self, value: str) -> KeyInfo:
        self.namespace = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> KeyInfo:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> KeyInfo:
        self.updated_at = value
        return self

    def with_value(self, value: str) -> KeyInfo:
        self.value = value
        return self

    def with_acquire_order_no(self, value: str) -> KeyInfo:
        self.acquire_order_no = value
        return self

    def with_acquire_user_id(self, value: str) -> KeyInfo:
        self.acquire_user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "key_file"):
            result["keyFile"] = str(self.key_file)
        elif include_empty:
            result["keyFile"] = ""
        if hasattr(self, "key_group_id"):
            result["keyGroupId"] = str(self.key_group_id)
        elif include_empty:
            result["keyGroupId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "value"):
            result["value"] = str(self.value)
        elif include_empty:
            result["value"] = ""
        if hasattr(self, "acquire_order_no"):
            result["acquireOrderNo"] = str(self.acquire_order_no)
        elif include_empty:
            result["acquireOrderNo"] = ""
        if hasattr(self, "acquire_user_id"):
            result["acquireUserId"] = str(self.acquire_user_id)
        elif include_empty:
            result["acquireUserId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        id_: str,
        key_file: str,
        key_group_id: str,
        namespace: str,
        status: Union[str, StatusEnum],
        updated_at: str,
        value: str,
        acquire_order_no: Optional[str] = None,
        acquire_user_id: Optional[str] = None,
    ) -> KeyInfo:
        instance = cls()
        instance.created_at = created_at
        instance.id_ = id_
        instance.key_file = key_file
        instance.key_group_id = key_group_id
        instance.namespace = namespace
        instance.status = status
        instance.updated_at = updated_at
        instance.value = value
        if acquire_order_no is not None:
            instance.acquire_order_no = acquire_order_no
        if acquire_user_id is not None:
            instance.acquire_user_id = acquire_user_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> KeyInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "keyFile" in dict_ and dict_["keyFile"] is not None:
            instance.key_file = str(dict_["keyFile"])
        elif include_empty:
            instance.key_file = ""
        if "keyGroupId" in dict_ and dict_["keyGroupId"] is not None:
            instance.key_group_id = str(dict_["keyGroupId"])
        elif include_empty:
            instance.key_group_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "value" in dict_ and dict_["value"] is not None:
            instance.value = str(dict_["value"])
        elif include_empty:
            instance.value = ""
        if "acquireOrderNo" in dict_ and dict_["acquireOrderNo"] is not None:
            instance.acquire_order_no = str(dict_["acquireOrderNo"])
        elif include_empty:
            instance.acquire_order_no = ""
        if "acquireUserId" in dict_ and dict_["acquireUserId"] is not None:
            instance.acquire_user_id = str(dict_["acquireUserId"])
        elif include_empty:
            instance.acquire_user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, KeyInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[KeyInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[KeyInfo, List[KeyInfo], Dict[Any, KeyInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "id": "id_",
            "keyFile": "key_file",
            "keyGroupId": "key_group_id",
            "namespace": "namespace",
            "status": "status",
            "updatedAt": "updated_at",
            "value": "value",
            "acquireOrderNo": "acquire_order_no",
            "acquireUserId": "acquire_user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "id": True,
            "keyFile": True,
            "keyGroupId": True,
            "namespace": True,
            "status": True,
            "updatedAt": True,
            "value": True,
            "acquireOrderNo": False,
            "acquireUserId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["ACQUIRED", "ACTIVE"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import HTTPValidationError
from ..models import TelemetryBody

from ..operations.gametelemetry_operations import (
    ProtectedGetPlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimeGet,
)
from ..operations.gametelemetry_operations import (
    ProtectedSaveEventsGameTelemetryV1ProtectedEventsPost,
)
from ..operations.gametelemetry_operations import (
    ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut,
)


@same_doc_as(ProtectedGetPlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimeGet)
def protected_get_playtime_game_telemetry_v1_protected_steam_ids_steam_id_playtime_get(
    steam_id: str,
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = (
        ProtectedGetPlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimeGet.create(
            steam_id=steam_id,
            cookie=cookie,
        )
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ProtectedGetPlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimeGet)
async def protected_get_playtime_game_telemetry_v1_protected_steam_ids_steam_id_playtime_get_async(
    steam_id: str,
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = (
        ProtectedGetPlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimeGet.create(
            steam_id=steam_id,
            cookie=cookie,
        )
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ProtectedSaveEventsGameTelemetryV1ProtectedEventsPost)
def protected_save_events_game_telemetry_v1_protected_events_post(
    body: List[TelemetryBody],
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = ProtectedSaveEventsGameTelemetryV1ProtectedEventsPost.create(
        body=body,
        cookie=cookie,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ProtectedSaveEventsGameTelemetryV1ProtectedEventsPost)
async def protected_save_events_game_telemetry_v1_protected_events_post_async(
    body: List[TelemetryBody],
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = ProtectedSaveEventsGameTelemetryV1ProtectedEventsPost.create(
        body=body,
        cookie=cookie,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(
    ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut
)
def protected_update_playtime_game_telemetry_v1_protected_steam_ids_steam_id_playtime_playtime_put(
    playtime: str,
    steam_id: str,
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut.create(
        playtime=playtime,
        steam_id=steam_id,
        cookie=cookie,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(
    ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut
)
async def protected_update_playtime_game_telemetry_v1_protected_steam_ids_steam_id_playtime_playtime_put_async(
    playtime: str,
    steam_id: str,
    cookie: Optional[Union[str, HeaderStr]] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut.create(
        playtime=playtime,
        steam_id=steam_id,
        cookie=cookie,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.14.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
import re
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class AppleIAPReceipt(Model):
    """Apple IAP receipt (AppleIAPReceipt)

    Properties:
        product_id: (productId) REQUIRED str

        receipt_data: (receiptData) REQUIRED str

        transaction_id: (transactionId) REQUIRED str

        exclude_old_transactions: (excludeOldTransactions) OPTIONAL bool

        language: (language) OPTIONAL str

        region: (region) OPTIONAL str
    """

    # region fields

    product_id: str  # REQUIRED
    receipt_data: str  # REQUIRED
    transaction_id: str  # REQUIRED
    exclude_old_transactions: bool  # OPTIONAL
    language: str  # OPTIONAL
    region: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_product_id(self, value: str) -> AppleIAPReceipt:
        self.product_id = value
        return self

    def with_receipt_data(self, value: str) -> AppleIAPReceipt:
        self.receipt_data = value
        return self

    def with_transaction_id(self, value: str) -> AppleIAPReceipt:
        self.transaction_id = value
        return self

    def with_exclude_old_transactions(self, value: bool) -> AppleIAPReceipt:
        self.exclude_old_transactions = value
        return self

    def with_language(self, value: str) -> AppleIAPReceipt:
        self.language = value
        return self

    def with_region(self, value: str) -> AppleIAPReceipt:
        self.region = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "product_id"):
            result["productId"] = str(self.product_id)
        elif include_empty:
            result["productId"] = ""
        if hasattr(self, "receipt_data"):
            result["receiptData"] = str(self.receipt_data)
        elif include_empty:
            result["receiptData"] = ""
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "exclude_old_transactions"):
            result["excludeOldTransactions"] = bool(self.exclude_old_transactions)
        elif include_empty:
            result["excludeOldTransactions"] = False
        if hasattr(self, "language"):
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        product_id: str,
        receipt_data: str,
        transaction_id: str,
        exclude_old_transactions: Optional[bool] = None,
        language: Optional[str] = None,
        region: Optional[str] = None,
    ) -> AppleIAPReceipt:
        instance = cls()
        instance.product_id = product_id
        instance.receipt_data = receipt_data
        instance.transaction_id = transaction_id
        if exclude_old_transactions is not None:
            instance.exclude_old_transactions = exclude_old_transactions
        if language is not None:
            instance.language = language
        if region is not None:
            instance.region = region
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AppleIAPReceipt:
        instance = cls()
        if not dict_:
            return instance
        if "productId" in dict_ and dict_["productId"] is not None:
            instance.product_id = str(dict_["productId"])
        elif include_empty:
            instance.product_id = ""
        if "receiptData" in dict_ and dict_["receiptData"] is not None:
            instance.receipt_data = str(dict_["receiptData"])
        elif include_empty:
            instance.receipt_data = ""
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if (
            "excludeOldTransactions" in dict_
            and dict_["excludeOldTransactions"] is not None
        ):
            instance.exclude_old_transactions = bool(dict_["excludeOldTransactions"])
        elif include_empty:
            instance.exclude_old_transactions = False
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AppleIAPReceipt]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AppleIAPReceipt]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[AppleIAPReceipt, List[AppleIAPReceipt], Dict[Any, AppleIAPReceipt]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "productId": "product_id",
            "receiptData": "receipt_data",
            "transactionId": "transaction_id",
            "excludeOldTransactions": "exclude_old_transactions",
            "language": "language",
            "region": "region",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "productId": True,
            "receiptData": True,
            "transactionId": True,
            "excludeOldTransactions": False,
            "language": False,
            "region": False,
        }

    @staticmethod
    def get_pattern_map() -> Dict[str, re.Pattern]:
        return {
            "language": re.compile(
                r"^[A-Za-z]{2,4}([_-][A-Za-z]{4})?([_-]([A-Za-z]{2}|[0-9]{3}))?$"
            ),
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Platform Service (4.14.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import CheckoutConfig
from ...models import ErrorEntity
from ...models import PaymentMerchantConfigInfo


class UpdateCheckoutConfig(Operation):
    """Update checkout.com config (updateCheckoutConfig)

    Update checkout.com config.
    Other detail info:

      * Required permission : resource="ADMIN:PAYMENT:CONFIG", action=4 (UPDATE)
      *  Returns : updated payment merchant config

    Required Permission(s):
        - ADMIN:PAYMENT:CONFIG [UPDATE]

    Properties:
        url: /platform/admin/payment/config/merchant/{id}/checkoutconfig

        method: PUT

        tags: ["PaymentConfig"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        body: (body) OPTIONAL CheckoutConfig in body

        id_: (id) REQUIRED str in path

        sandbox: (sandbox) OPTIONAL bool in query

        validate: (validate) OPTIONAL bool in query

    Responses:
        200: OK - PaymentMerchantConfigInfo (successful operation)

        404: Not Found - ErrorEntity (33242: Payment merchant config [{id}] does not exist | 33221: Update [{paymentProvider}] config in payment merchant config [{id}] failed with message [{errMsg}])
    """

    # region fields

    _url: str = "/platform/admin/payment/config/merchant/{id}/checkoutconfig"
    _method: str = "PUT"
    _consumes: List[str] = ["application/json"]
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    body: CheckoutConfig  # OPTIONAL in [body]
    id_: str  # REQUIRED in [path]
    sandbox: bool  # OPTIONAL in [query]
    validate: bool  # OPTIONAL in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "body": self.get_body_params(),
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_body_params(self) -> Any:
        if not hasattr(self, "body") or self.body is None:
            return None
        return self.body.to_dict()

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "id_"):
            result["id"] = self.id_
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "sandbox"):
            result["sandbox"] = self.sandbox
        if hasattr(self, "validate"):
            result["validate"] = self.validate
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_body(self, value: CheckoutConfig) -> UpdateCheckoutConfig:
        self.body = value
        return self

    def with_id_(self, value: str) -> UpdateCheckoutConfig:
        self.id_ = value
        return self

    def with_sandbox(self, value: bool) -> UpdateCheckoutConfig:
        self.sandbox = value
        return self

    def with_validate(self, value: bool) -> UpdateCheckoutConfig:
        self.validate = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body") and self.body:
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = CheckoutConfig()
        if hasattr(self, "id_") and self.id_:
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "sandbox") and self.sandbox:
            result["sandbox"] = bool(self.sandbox)
        elif include_empty:
            result["sandbox"] = False
        if hasattr(self, "validate") and self.validate:
            result["validate"] = bool(self.validate)
        elif include_empty:
            result["validate"] = False
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, PaymentMerchantConfigInfo], Union[None, ErrorEntity, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - PaymentMerchantConfigInfo (successful operation)

        404: Not Found - ErrorEntity (33242: Payment merchant config [{id}] does not exist | 33221: Update [{paymentProvider}] config in payment merchant config [{id}] failed with message [{errMsg}])

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return PaymentMerchantConfigInfo.create_from_dict(content), None
        if code == 404:
            return None, ErrorEntity.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        body: Optional[CheckoutConfig] = None,
        sandbox: Optional[bool] = None,
        validate: Optional[bool] = None,
    ) -> UpdateCheckoutConfig:
        instance = cls()
        instance.id_ = id_
        if body is not None:
            instance.body = body
        if sandbox is not None:
            instance.sandbox = sandbox
        if validate is not None:
            instance.validate = validate
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> UpdateCheckoutConfig:
        instance = cls()
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = CheckoutConfig.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = CheckoutConfig()
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "sandbox" in dict_ and dict_["sandbox"] is not None:
            instance.sandbox = bool(dict_["sandbox"])
        elif include_empty:
            instance.sandbox = False
        if "validate" in dict_ and dict_["validate"] is not None:
            instance.validate = bool(dict_["validate"])
        elif include_empty:
            instance.validate = False
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "id": "id_",
            "sandbox": "sandbox",
            "validate": "validate",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": False,
            "id": True,
            "sandbox": False,
            "validate": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Platform Service (4.14.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse
from .....core import StrEnum


class ItemTypeEnum(StrEnum):
    APP = "APP"
    BUNDLE = "BUNDLE"
    CODE = "CODE"
    COINS = "COINS"
    EXTENSION = "EXTENSION"
    INGAMEITEM = "INGAMEITEM"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SEASON = "SEASON"
    SUBSCRIPTION = "SUBSCRIPTION"


class DownloadInvoiceDetails(Operation):
    """Download invoice details (downloadInvoiceDetails)

    Download invoice details as a csv file.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:{namespace}:ORDER", action=2 (READ)
      *  Returns : invoice details csv file

    Required Permission(s):
        - ADMIN:NAMESPACE:{namespace}:ORDER [READ]

    Properties:
        url: /platform/admin/namespaces/{namespace}/invoice/details.csv

        method: GET

        tags: ["Invoice"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        feature: (feature) OPTIONAL str in query

        item_id: (itemId) OPTIONAL str in query

        item_type: (itemType) OPTIONAL Union[str, ItemTypeEnum] in query

        end_time: (endTime) REQUIRED str in query

        start_time: (startTime) REQUIRED str in query

    Responses:
        200: OK - (Successful operation)
    """

    # region fields

    _url: str = "/platform/admin/namespaces/{namespace}/invoice/details.csv"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["text/csv"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    namespace: str  # REQUIRED in [path]
    feature: str  # OPTIONAL in [query]
    item_id: str  # OPTIONAL in [query]
    item_type: Union[str, ItemTypeEnum]  # OPTIONAL in [query]
    end_time: str  # REQUIRED in [query]
    start_time: str  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "path": self.get_path_params(),
            "query": self.get_query_params(),
        }

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "namespace"):
            result["namespace"] = self.namespace
        return result

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "feature"):
            result["feature"] = self.feature
        if hasattr(self, "item_id"):
            result["itemId"] = self.item_id
        if hasattr(self, "item_type"):
            result["itemType"] = self.item_type
        if hasattr(self, "end_time"):
            result["endTime"] = self.end_time
        if hasattr(self, "start_time"):
            result["startTime"] = self.start_time
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_namespace(self, value: str) -> DownloadInvoiceDetails:
        self.namespace = value
        return self

    def with_feature(self, value: str) -> DownloadInvoiceDetails:
        self.feature = value
        return self

    def with_item_id(self, value: str) -> DownloadInvoiceDetails:
        self.item_id = value
        return self

    def with_item_type(self, value: Union[str, ItemTypeEnum]) -> DownloadInvoiceDetails:
        self.item_type = value
        return self

    def with_end_time(self, value: str) -> DownloadInvoiceDetails:
        self.end_time = value
        return self

    def with_start_time(self, value: str) -> DownloadInvoiceDetails:
        self.start_time = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "namespace") and self.namespace:
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "feature") and self.feature:
            result["feature"] = str(self.feature)
        elif include_empty:
            result["feature"] = ""
        if hasattr(self, "item_id") and self.item_id:
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_type") and self.item_type:
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = Union[str, ItemTypeEnum]()
        if hasattr(self, "end_time") and self.end_time:
            result["endTime"] = str(self.end_time)
        elif include_empty:
            result["endTime"] = ""
        if hasattr(self, "start_time") and self.start_time:
            result["startTime"] = str(self.start_time)
        elif include_empty:
            result["startTime"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[Union[None, HttpResponse], Union[None, HttpResponse]]:
        """Parse the given response.

        200: OK - (Successful operation)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return HttpResponse.create(code, "OK"), None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        namespace: str,
        end_time: str,
        start_time: str,
        feature: Optional[str] = None,
        item_id: Optional[str] = None,
        item_type: Optional[Union[str, ItemTypeEnum]] = None,
    ) -> DownloadInvoiceDetails:
        instance = cls()
        instance.namespace = namespace
        instance.end_time = end_time
        instance.start_time = start_time
        if feature is not None:
            instance.feature = feature
        if item_id is not None:
            instance.item_id = item_id
        if item_type is not None:
            instance.item_type = item_type
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DownloadInvoiceDetails:
        instance = cls()
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "feature" in dict_ and dict_["feature"] is not None:
            instance.feature = str(dict_["feature"])
        elif include_empty:
            instance.feature = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = Union[str, ItemTypeEnum]()
        if "endTime" in dict_ and dict_["endTime"] is not None:
            instance.end_time = str(dict_["endTime"])
        elif include_empty:
            instance.end_time = ""
        if "startTime" in dict_ and dict_["startTime"] is not None:
            instance.start_time = str(dict_["startTime"])
        elif include_empty:
            instance.start_time = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "namespace": "namespace",
            "feature": "feature",
            "itemId": "item_id",
            "itemType": "item_type",
            "endTime": "end_time",
            "startTime": "start_time",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "namespace": True,
            "feature": False,
            "itemId": False,
            "itemType": False,
            "endTime": True,
            "startTime": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemType": [
                "APP",
                "BUNDLE",
                "CODE",
                "COINS",
                "EXTENSION",
                "INGAMEITEM",
                "MEDIA",
                "OPTIONBOX",
                "SEASON",
                "SUBSCRIPTION",
            ],  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Platform Service."""

__version__ = "4.14.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .check_event_condition import CheckEventCondition
from .create_reward import CreateReward
from .delete_reward import DeleteReward
from .delete_reward_condition_record import DeleteRewardConditionRecord
from .export_rewards import ExportRewards
from .get_reward import GetReward
from .get_reward_1 import GetReward1
from .get_reward_by_code import GetRewardByCode
from .import_rewards import ImportRewards
from .query_rewards import QueryRewards
from .query_rewards import SortByEnum as QueryRewardsSortByEnum
from .query_rewards_1 import QueryRewards1
from .query_rewards_1 import SortByEnum as QueryRewards1SortByEnum
from .update_reward import UpdateReward



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Ugc Service."""

__version__ = "2.5.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_delete_all_user_channels import AdminDeleteAllUserChannels
from .admin_delete_all_user_contents import AdminDeleteAllUserContents
from .admin_delete_all_user_group import AdminDeleteAllUserGroup
from .admin_delete_all_user_states import AdminDeleteAllUserStates
from .delete_all_user_channel import DeleteAllUserChannel
from .delete_all_user_contents import DeleteAllUserContents
from .delete_all_user_group import DeleteAllUserGroup
from .delete_all_user_states import DeleteAllUserStates



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsCreatePodConfigRequest
from ..models import ModelsListPodConfigResponse
from ..models import ModelsPodConfigRecord
from ..models import ModelsUpdatePodConfigRequest
from ..models import ResponseError

from ..operations.pod_config import CreatePodConfig
from ..operations.pod_config import DeletePodConfig
from ..operations.pod_config import GetAllPodConfig
from ..operations.pod_config import GetPodConfig
from ..operations.pod_config import UpdatePodConfig


@same_doc_as(CreatePodConfig)
def create_pod_config(
    body: ModelsCreatePodConfigRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePodConfig.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreatePodConfig)
async def create_pod_config_async(
    body: ModelsCreatePodConfigRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreatePodConfig.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeletePodConfig)
def delete_pod_config(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePodConfig.create(
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeletePodConfig)
async def delete_pod_config_async(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePodConfig.create(
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAllPodConfig)
def get_all_pod_config(
    count: int,
    offset: int,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllPodConfig.create(
        count=count,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAllPodConfig)
async def get_all_pod_config_async(
    count: int,
    offset: int,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAllPodConfig.create(
        count=count,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPodConfig)
def get_pod_config(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPodConfig.create(
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPodConfig)
async def get_pod_config_async(
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPodConfig.create(
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdatePodConfig)
def update_pod_config(
    body: ModelsUpdatePodConfigRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdatePodConfig.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdatePodConfig)
async def update_pod_config_async(
    body: ModelsUpdatePodConfigRequest,
    name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdatePodConfig.create(
        body=body,
        name=name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Iam Service (5.16.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ModelBackupCodesResponseV4(Model):
    """Model backup codes response V4 (model.BackupCodesResponseV4)

    Properties:
        generated_at: (generatedAt) REQUIRED int

        valid_codes: (validCodes) REQUIRED List[str]

        invalid_codes: (invalidCodes) OPTIONAL List[str]
    """

    # region fields

    generated_at: int  # REQUIRED
    valid_codes: List[str]  # REQUIRED
    invalid_codes: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_generated_at(self, value: int) -> ModelBackupCodesResponseV4:
        self.generated_at = value
        return self

    def with_valid_codes(self, value: List[str]) -> ModelBackupCodesResponseV4:
        self.valid_codes = value
        return self

    def with_invalid_codes(self, value: List[str]) -> ModelBackupCodesResponseV4:
        self.invalid_codes = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "generated_at"):
            result["generatedAt"] = int(self.generated_at)
        elif include_empty:
            result["generatedAt"] = 0
        if hasattr(self, "valid_codes"):
            result["validCodes"] = [str(i0) for i0 in self.valid_codes]
        elif include_empty:
            result["validCodes"] = []
        if hasattr(self, "invalid_codes"):
            result["invalidCodes"] = [str(i0) for i0 in self.invalid_codes]
        elif include_empty:
            result["invalidCodes"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        generated_at: int,
        valid_codes: List[str],
        invalid_codes: Optional[List[str]] = None,
    ) -> ModelBackupCodesResponseV4:
        instance = cls()
        instance.generated_at = generated_at
        instance.valid_codes = valid_codes
        if invalid_codes is not None:
            instance.invalid_codes = invalid_codes
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelBackupCodesResponseV4:
        instance = cls()
        if not dict_:
            return instance
        if "generatedAt" in dict_ and dict_["generatedAt"] is not None:
            instance.generated_at = int(dict_["generatedAt"])
        elif include_empty:
            instance.generated_at = 0
        if "validCodes" in dict_ and dict_["validCodes"] is not None:
            instance.valid_codes = [str(i0) for i0 in dict_["validCodes"]]
        elif include_empty:
            instance.valid_codes = []
        if "invalidCodes" in dict_ and dict_["invalidCodes"] is not None:
            instance.invalid_codes = [str(i0) for i0 in dict_["invalidCodes"]]
        elif include_empty:
            instance.invalid_codes = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelBackupCodesResponseV4]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelBackupCodesResponseV4]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelBackupCodesResponseV4,
        List[ModelBackupCodesResponseV4],
        Dict[Any, ModelBackupCodesResponseV4],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "generatedAt": "generated_at",
            "validCodes": "valid_codes",
            "invalidCodes": "invalid_codes",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "generatedAt": True,
            "validCodes": True,
            "invalidCodes": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import StoreBackupInfo
from ..models import StoreCreate
from ..models import StoreInfo
from ..models import StoreUpdate
from ..models import ValidationErrorEntity

from ..operations.store import CloneStore
from ..operations.store import CreateStore
from ..operations.store import DeletePublishedStore
from ..operations.store import DeleteStore
from ..operations.store import ExportStore
from ..operations.store import GetPublishedStore
from ..operations.store import GetPublishedStoreBackup
from ..operations.store import GetStore
from ..operations.store import ImportStore
from ..operations.store import ListStores
from ..operations.store import PublicListStores
from ..operations.store import RollbackPublishedStore
from ..operations.store import UpdateStore


@same_doc_as(CloneStore)
def clone_store(
    store_id: str,
    target_store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CloneStore.create(
        store_id=store_id,
        target_store_id=target_store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CloneStore)
async def clone_store_async(
    store_id: str,
    target_store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CloneStore.create(
        store_id=store_id,
        target_store_id=target_store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateStore)
def create_store(
    body: Optional[StoreCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateStore.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateStore)
async def create_store_async(
    body: Optional[StoreCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateStore.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeletePublishedStore)
def delete_published_store(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePublishedStore.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeletePublishedStore)
async def delete_published_store_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeletePublishedStore.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteStore)
def delete_store(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteStore)
async def delete_store_async(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ExportStore)
def export_store(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ExportStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ExportStore)
async def export_store_async(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ExportStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPublishedStore)
def get_published_store(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPublishedStore.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPublishedStore)
async def get_published_store_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPublishedStore.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPublishedStoreBackup)
def get_published_store_backup(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPublishedStoreBackup.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPublishedStoreBackup)
async def get_published_store_backup_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPublishedStoreBackup.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetStore)
def get_store(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetStore)
async def get_store_async(
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetStore.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ImportStore)
def import_store(
    file: Optional[Any] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImportStore.create(
        file=file,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ImportStore)
async def import_store_async(
    file: Optional[Any] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImportStore.create(
        file=file,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListStores)
def list_stores(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListStores.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListStores)
async def list_stores_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListStores.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicListStores)
def public_list_stores(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListStores.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListStores)
async def public_list_stores_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListStores.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RollbackPublishedStore)
def rollback_published_store(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RollbackPublishedStore.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RollbackPublishedStore)
async def rollback_published_store_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = RollbackPublishedStore.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateStore)
def update_store(
    store_id: str,
    body: Optional[StoreUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateStore.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateStore)
async def update_store_async(
    store_id: str,
    body: Optional[StoreUpdate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateStore.create(
        store_id=store_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Dsm Controller Service (3.6.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from .utils import randomize

from ..api.dsmc.models import LogAppMessageDeclaration
from ..api.dsmc.models import ModelsClaimSessionRequest
from ..api.dsmc.models import ModelsCountServerResponse
from ..api.dsmc.models import ModelsCountSessionResponse
from ..api.dsmc.models import ModelsCreateDSMConfigRequest
from ..api.dsmc.models import ModelsCreateDeploymentOverrideRequest
from ..api.dsmc.models import ModelsCreateDeploymentRequest
from ..api.dsmc.models import ModelsCreateImagePatchRequest
from ..api.dsmc.models import ModelsCreateImageRequest
from ..api.dsmc.models import ModelsCreatePodConfigRequest
from ..api.dsmc.models import ModelsCreatePortRequest
from ..api.dsmc.models import ModelsCreateRegionOverrideRequest
from ..api.dsmc.models import ModelsCreateSessionRequest
from ..api.dsmc.models import ModelsDSMConfigExport
from ..api.dsmc.models import ModelsDSMConfigRecord
from ..api.dsmc.models import ModelsDefaultProvider
from ..api.dsmc.models import ModelsDeploymentConfigOverride
from ..api.dsmc.models import ModelsDeploymentWithOverride
from ..api.dsmc.models import ModelsDeregisterLocalServerRequest
from ..api.dsmc.models import ModelsDetailedCountServerResponse
from ..api.dsmc.models import ModelsGetImageDetailResponse
from ..api.dsmc.models import ModelsGetImageLimitResponse
from ..api.dsmc.models import ModelsGetImageLimitResponseData
from ..api.dsmc.models import ModelsGetImagePatchDetailResponse
from ..api.dsmc.models import ModelsImageRecord
from ..api.dsmc.models import ModelsImageRecordUpdate
from ..api.dsmc.models import ModelsImportResponse
from ..api.dsmc.models import ModelsListConfigResponse
from ..api.dsmc.models import ModelsListDeploymentResponse
from ..api.dsmc.models import ModelsListImagePatchesResponse
from ..api.dsmc.models import ModelsListImageResponse
from ..api.dsmc.models import ModelsListPodConfigResponse
from ..api.dsmc.models import ModelsListServerResponse
from ..api.dsmc.models import ModelsListSessionResponse
from ..api.dsmc.models import ModelsMatchResultNotificationPayload
from ..api.dsmc.models import ModelsPagingCursor
from ..api.dsmc.models import ModelsPatchImageRecord
from ..api.dsmc.models import ModelsPodConfigRecord
from ..api.dsmc.models import ModelsPodCountConfigOverride
from ..api.dsmc.models import ModelsRegisterLocalServerRequest
from ..api.dsmc.models import ModelsRegisterServerRequest
from ..api.dsmc.models import ModelsRequestMatchMember
from ..api.dsmc.models import ModelsRequestMatchParty
from ..api.dsmc.models import ModelsRequestMatchingAlly
from ..api.dsmc.models import ModelsServer
from ..api.dsmc.models import ModelsServerLogs
from ..api.dsmc.models import ModelsServerSessionResponse
from ..api.dsmc.models import ModelsSession
from ..api.dsmc.models import ModelsSessionResponse
from ..api.dsmc.models import ModelsShutdownServerRequest
from ..api.dsmc.models import ModelsStatusHistory
from ..api.dsmc.models import ModelsUpdateDSMConfigRequest
from ..api.dsmc.models import ModelsUpdateDeploymentOverrideRequest
from ..api.dsmc.models import ModelsUpdateDeploymentRequest
from ..api.dsmc.models import ModelsUpdatePodConfigRequest
from ..api.dsmc.models import ModelsUpdatePortRequest
from ..api.dsmc.models import ModelsUpdateRegionOverrideRequest
from ..api.dsmc.models import ResponseError


def create_log_app_message_declaration_example() -> LogAppMessageDeclaration:
    instance = LogAppMessageDeclaration()
    instance.attributes = [randomize()]
    instance.code = randomize()
    instance.code_name = randomize()
    instance.section = randomize()
    instance.service = randomize()
    instance.text = randomize()
    return instance


def create_models_claim_session_request_example() -> ModelsClaimSessionRequest:
    instance = ModelsClaimSessionRequest()
    instance.session_id = randomize("uid")
    return instance


def create_models_count_server_response_example() -> ModelsCountServerResponse:
    instance = ModelsCountServerResponse()
    instance.count = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_count_session_response_example() -> ModelsCountSessionResponse:
    instance = ModelsCountSessionResponse()
    instance.count = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_deployment_override_request_example() -> ModelsCreateDeploymentOverrideRequest:
    instance = ModelsCreateDeploymentOverrideRequest()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.region_overrides = {}
    instance.regions = [randomize()]
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_deployment_request_example() -> ModelsCreateDeploymentRequest:
    instance = ModelsCreateDeploymentRequest()
    instance.allow_version_override = randomize("bool")
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.overrides = {}
    instance.region_overrides = {}
    instance.regions = [randomize()]
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_dsm_config_request_example() -> ModelsCreateDSMConfigRequest:
    instance = ModelsCreateDSMConfigRequest()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    instance.default_version = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.protocol = randomize()
    instance.providers = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unreachable_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_image_patch_request_example() -> ModelsCreateImagePatchRequest:
    instance = ModelsCreateImagePatchRequest()
    instance.artifact_path = randomize()
    instance.docker_path = randomize()
    instance.image = randomize()
    instance.image_size = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.patch_version = randomize()
    instance.persistent = randomize("bool")
    instance.version = randomize("version")
    return instance


def create_models_create_image_request_example() -> ModelsCreateImageRequest:
    instance = ModelsCreateImageRequest()
    instance.artifact_path = randomize()
    instance.docker_path = randomize()
    instance.image = randomize()
    instance.image_size = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.persistent = randomize("bool")
    instance.version = randomize("version")
    return instance


def create_models_create_pod_config_request_example() -> ModelsCreatePodConfigRequest:
    instance = ModelsCreatePodConfigRequest()
    instance.cpu_limit = randomize("int", min_val=1, max_val=1000)
    instance.mem_limit = randomize("int", min_val=1, max_val=1000)
    instance.params = randomize()
    return instance


def create_models_create_port_request_example() -> ModelsCreatePortRequest:
    instance = ModelsCreatePortRequest()
    instance.port = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_create_region_override_request_example() -> ModelsCreateRegionOverrideRequest:
    instance = ModelsCreateRegionOverrideRequest()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    return instance


def create_models_create_session_request_example() -> ModelsCreateSessionRequest:
    instance = ModelsCreateSessionRequest()
    instance.client_version = randomize()
    instance.configuration = randomize()
    instance.deployment = randomize()
    instance.game_mode = randomize()
    instance.matching_allies = [create_models_request_matching_ally_example()]
    instance.namespace = randomize("slug")
    instance.pod_name = randomize()
    instance.region = randomize()
    instance.session_id = randomize("uid")
    instance.notification_payload = (
        create_models_match_result_notification_payload_example()
    )
    return instance


def create_models_default_provider_example() -> ModelsDefaultProvider:
    instance = ModelsDefaultProvider()
    instance.provider_name = randomize()
    return instance


def create_models_deployment_config_override_example() -> ModelsDeploymentConfigOverride:
    instance = ModelsDeploymentConfigOverride()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.region_overrides = {}
    instance.regions = [randomize()]
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_deployment_with_override_example() -> ModelsDeploymentWithOverride:
    instance = ModelsDeploymentWithOverride()
    instance.allow_version_override = randomize("bool")
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.created_at = randomize("date")
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.modified_by = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.overrides = {}
    instance.region_overrides = {}
    instance.regions = [randomize()]
    instance.unlimited = randomize("bool")
    instance.updated_at = randomize("date")
    instance.use_buffer_percent = randomize("bool")
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_deregister_local_server_request_example() -> ModelsDeregisterLocalServerRequest:
    instance = ModelsDeregisterLocalServerRequest()
    instance.name = randomize()
    return instance


def create_models_detailed_count_server_response_example() -> ModelsDetailedCountServerResponse:
    instance = ModelsDetailedCountServerResponse()
    instance.busy_count = randomize("int", min_val=1, max_val=1000)
    instance.creating_count = randomize("int", min_val=1, max_val=1000)
    instance.ready_count = randomize("int", min_val=1, max_val=1000)
    instance.unreachable_count = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_dsm_config_export_example() -> ModelsDSMConfigExport:
    instance = ModelsDSMConfigExport()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.created_at = randomize("date")
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    instance.default_version = randomize()
    instance.deployments = [create_models_deployment_with_override_example()]
    instance.images = [create_models_image_record_example()]
    instance.namespace = randomize("slug")
    instance.pod_configs = [create_models_pod_config_record_example()]
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.protocol = randomize()
    instance.providers = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unreachable_timeout = randomize("int", min_val=1, max_val=1000)
    instance.updated_at = randomize("date")
    return instance


def create_models_dsm_config_record_example() -> ModelsDSMConfigRecord:
    instance = ModelsDSMConfigRecord()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.created_at = randomize("date")
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    instance.default_version = randomize()
    instance.modified_by = randomize()
    instance.namespace = randomize("slug")
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.protocol = randomize()
    instance.providers = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unreachable_timeout = randomize("int", min_val=1, max_val=1000)
    instance.updated_at = randomize("date")
    return instance


def create_models_get_image_detail_response_example() -> ModelsGetImageDetailResponse:
    instance = ModelsGetImageDetailResponse()
    instance.data = create_models_image_record_example()
    return instance


def create_models_get_image_limit_response_example() -> ModelsGetImageLimitResponse:
    instance = ModelsGetImageLimitResponse()
    instance.data = create_models_get_image_limit_response_data_example()
    return instance


def create_models_get_image_limit_response_data_example() -> ModelsGetImageLimitResponseData:
    instance = ModelsGetImageLimitResponseData()
    instance.image_count = randomize("int", min_val=1, max_val=1000)
    instance.image_limit = randomize("int", min_val=1, max_val=1000)
    instance.non_persistent_image_number = randomize("int", min_val=1, max_val=1000)
    instance.non_persistent_image_used_in_deployment_number = randomize(
        "int", min_val=1, max_val=1000
    )
    return instance


def create_models_get_image_patch_detail_response_example() -> ModelsGetImagePatchDetailResponse:
    instance = ModelsGetImagePatchDetailResponse()
    instance.data = create_models_patch_image_record_example()
    return instance


def create_models_image_record_example() -> ModelsImageRecord:
    instance = ModelsImageRecord()
    instance.artifact_path = randomize()
    instance.created_at = randomize("date")
    instance.docker_path = randomize()
    instance.image = randomize()
    instance.image_size = randomize("int", min_val=1, max_val=1000)
    instance.modified_by = randomize()
    instance.namespace = randomize("slug")
    instance.persistent = randomize("bool")
    instance.updated_at = randomize("date")
    instance.version = randomize("version")
    return instance


def create_models_image_record_update_example() -> ModelsImageRecordUpdate:
    instance = ModelsImageRecordUpdate()
    instance.artifact_path = randomize()
    instance.image = randomize()
    instance.namespace = randomize("slug")
    instance.persistent = randomize("bool")
    instance.version = randomize("version")
    return instance


def create_models_import_response_example() -> ModelsImportResponse:
    instance = ModelsImportResponse()
    instance.affected = [randomize()]
    instance.failed = [randomize()]
    return instance


def create_models_list_config_response_example() -> ModelsListConfigResponse:
    instance = ModelsListConfigResponse()
    instance.configs = [create_models_dsm_config_record_example()]
    return instance


def create_models_list_deployment_response_example() -> ModelsListDeploymentResponse:
    instance = ModelsListDeploymentResponse()
    instance.deployments = [create_models_deployment_with_override_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_list_image_patches_response_example() -> ModelsListImagePatchesResponse:
    instance = ModelsListImagePatchesResponse()
    instance.images = [create_models_patch_image_record_example()]
    return instance


def create_models_list_image_response_example() -> ModelsListImageResponse:
    instance = ModelsListImageResponse()
    instance.images = [create_models_image_record_example()]
    instance.paging = create_models_paging_cursor_example()
    return instance


def create_models_list_pod_config_response_example() -> ModelsListPodConfigResponse:
    instance = ModelsListPodConfigResponse()
    instance.paging = create_models_paging_cursor_example()
    instance.pod_configs = [create_models_pod_config_record_example()]
    return instance


def create_models_list_server_response_example() -> ModelsListServerResponse:
    instance = ModelsListServerResponse()
    instance.paging = create_models_paging_cursor_example()
    instance.servers = [create_models_server_example()]
    return instance


def create_models_list_session_response_example() -> ModelsListSessionResponse:
    instance = ModelsListSessionResponse()
    instance.paging = create_models_paging_cursor_example()
    instance.sessions = [create_models_session_example()]
    return instance


def create_models_match_result_notification_payload_example() -> ModelsMatchResultNotificationPayload:
    instance = ModelsMatchResultNotificationPayload()
    return instance


def create_models_paging_cursor_example() -> ModelsPagingCursor:
    instance = ModelsPagingCursor()
    instance.next_ = randomize()
    instance.previous = randomize()
    return instance


def create_models_patch_image_record_example() -> ModelsPatchImageRecord:
    instance = ModelsPatchImageRecord()
    instance.artifact_path = randomize()
    instance.created_at = randomize("date")
    instance.docker_path = randomize()
    instance.image = randomize()
    instance.image_size = randomize("int", min_val=1, max_val=1000)
    instance.modified_by = randomize()
    instance.namespace = randomize("slug")
    instance.patch_version = randomize()
    instance.persistent = randomize("bool")
    instance.updated_at = randomize("date")
    instance.version = randomize("version")
    return instance


def create_models_pod_config_record_example() -> ModelsPodConfigRecord:
    instance = ModelsPodConfigRecord()
    instance.cpu_limit = randomize("int", min_val=1, max_val=1000)
    instance.created_at = randomize("date")
    instance.mem_limit = randomize("int", min_val=1, max_val=1000)
    instance.modified_by = randomize()
    instance.name = randomize()
    instance.namespace = randomize("slug")
    instance.params = randomize()
    instance.updated_at = randomize("date")
    return instance


def create_models_pod_count_config_override_example() -> ModelsPodCountConfigOverride:
    instance = ModelsPodCountConfigOverride()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    return instance


def create_models_register_local_server_request_example() -> ModelsRegisterLocalServerRequest:
    instance = ModelsRegisterLocalServerRequest()
    instance.custom_attribute = randomize()
    instance.ip = randomize()
    instance.name = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_register_server_request_example() -> ModelsRegisterServerRequest:
    instance = ModelsRegisterServerRequest()
    instance.custom_attribute = randomize()
    instance.pod_name = randomize()
    return instance


def create_models_request_match_member_example() -> ModelsRequestMatchMember:
    instance = ModelsRequestMatchMember()
    instance.user_id = randomize("uid")
    return instance


def create_models_request_match_party_example() -> ModelsRequestMatchParty:
    instance = ModelsRequestMatchParty()
    instance.party_attributes = {randomize(): randomize()}
    instance.party_id = randomize("uid")
    instance.party_members = [create_models_request_match_member_example()]
    return instance


def create_models_request_matching_ally_example() -> ModelsRequestMatchingAlly:
    instance = ModelsRequestMatchingAlly()
    instance.matching_parties = [create_models_request_match_party_example()]
    return instance


def create_models_server_example() -> ModelsServer:
    instance = ModelsServer()
    instance.allocation_id = randomize()
    instance.alternate_ips = [randomize()]
    instance.cpu_limit = randomize("int", min_val=1, max_val=1000)
    instance.created_at = randomize("date")
    instance.custom_attribute = randomize()
    instance.deployment = randomize()
    instance.game_version = randomize("version")
    instance.image_version = randomize()
    instance.ip = randomize()
    instance.is_override_game_version = randomize("bool")
    instance.job_id = randomize()
    instance.last_update = randomize("date")
    instance.mem_limit = randomize("int", min_val=1, max_val=1000)
    instance.namespace = randomize("slug")
    instance.params = randomize()
    instance.pod_name = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.ports = {}
    instance.protocol = randomize()
    instance.provider = randomize()
    instance.region = randomize()
    instance.session_id = randomize("uid")
    instance.status = randomize()
    instance.status_history = [create_models_status_history_example()]
    return instance


def create_models_server_logs_example() -> ModelsServerLogs:
    instance = ModelsServerLogs()
    instance.logs = randomize()
    return instance


def create_models_server_session_response_example() -> ModelsServerSessionResponse:
    instance = ModelsServerSessionResponse()
    instance.session_id = randomize("uid")
    return instance


def create_models_session_example() -> ModelsSession:
    instance = ModelsSession()
    instance.id_ = randomize()
    instance.namespace = randomize("slug")
    instance.provider = randomize()
    instance.region = randomize()
    instance.server = create_models_server_example()
    return instance


def create_models_session_response_example() -> ModelsSessionResponse:
    instance = ModelsSessionResponse()
    instance.session = create_models_session_example()
    return instance


def create_models_shutdown_server_request_example() -> ModelsShutdownServerRequest:
    instance = ModelsShutdownServerRequest()
    instance.kill_me = randomize("bool")
    instance.pod_name = randomize()
    return instance


def create_models_status_history_example() -> ModelsStatusHistory:
    instance = ModelsStatusHistory()
    instance.status = randomize()
    instance.time_stamp = randomize("date")
    return instance


def create_models_update_deployment_override_request_example() -> ModelsUpdateDeploymentOverrideRequest:
    instance = ModelsUpdateDeploymentOverrideRequest()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.regions = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    return instance


def create_models_update_deployment_request_example() -> ModelsUpdateDeploymentRequest:
    instance = ModelsUpdateDeploymentRequest()
    instance.allow_version_override = randomize("bool")
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.configuration = randomize()
    instance.enable_region_overrides = randomize("bool")
    instance.game_version = randomize("version")
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.regions = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    return instance


def create_models_update_dsm_config_request_example() -> ModelsUpdateDSMConfigRequest:
    instance = ModelsUpdateDSMConfigRequest()
    instance.claim_timeout = randomize("int", min_val=1, max_val=1000)
    instance.creation_timeout = randomize("int", min_val=1, max_val=1000)
    instance.default_version = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    instance.protocol = randomize()
    instance.providers = [randomize()]
    instance.session_timeout = randomize("int", min_val=1, max_val=1000)
    instance.unreachable_timeout = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_pod_config_request_example() -> ModelsUpdatePodConfigRequest:
    instance = ModelsUpdatePodConfigRequest()
    instance.cpu_limit = randomize("int", min_val=1, max_val=1000)
    instance.mem_limit = randomize("int", min_val=1, max_val=1000)
    instance.name = randomize()
    instance.params = randomize()
    return instance


def create_models_update_port_request_example() -> ModelsUpdatePortRequest:
    instance = ModelsUpdatePortRequest()
    instance.name = randomize()
    instance.port = randomize("int", min_val=1, max_val=1000)
    return instance


def create_models_update_region_override_request_example() -> ModelsUpdateRegionOverrideRequest:
    instance = ModelsUpdateRegionOverrideRequest()
    instance.buffer_count = randomize("int", min_val=1, max_val=1000)
    instance.buffer_percent = randomize("int", min_val=1, max_val=1000)
    instance.max_count = randomize("int", min_val=1, max_val=1000)
    instance.min_count = randomize("int", min_val=1, max_val=1000)
    instance.unlimited = randomize("bool")
    instance.use_buffer_percent = randomize("bool")
    return instance


def create_response_error_example() -> ResponseError:
    instance = ResponseError()
    instance.error_code = randomize("int", min_val=1, max_val=1000)
    instance.error_message = randomize()
    return instance



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import Customization
from ..models import ErrorEntity
from ..models import PaymentMethod
from ..models import PaymentOrderDetails
from ..models import PaymentOrderPaidResult
from ..models import PaymentProcessResult
from ..models import PaymentToken
from ..models import PaymentUrl
from ..models import PaymentUrlCreate
from ..models import TaxResult

from ..operations.payment_station import GetPaymentCustomization
from ..operations.payment_station import GetPaymentCustomizationPaymentProviderEnum
from ..operations.payment_station import GetPaymentPublicConfig
from ..operations.payment_station import GetPaymentPublicConfigPaymentProviderEnum
from ..operations.payment_station import GetPaymentTaxValue
from ..operations.payment_station import GetPaymentTaxValuePaymentProviderEnum
from ..operations.payment_station import Pay
from ..operations.payment_station import PayPaymentProviderEnum
from ..operations.payment_station import PublicCheckPaymentOrderPaidStatus
from ..operations.payment_station import PublicGetPaymentMethods
from ..operations.payment_station import PublicGetPaymentUrl
from ..operations.payment_station import PublicGetQRCode
from ..operations.payment_station import PublicGetUnpaidPaymentOrder
from ..operations.payment_station import PublicNormalizePaymentReturnUrl
from ..operations.payment_station import (
    PublicNormalizePaymentReturnUrlPaymentProviderEnum,
)
from ..models import PaymentMethodPaymentProviderEnum
from ..models import PaymentUrlPaymentProviderEnum, PaymentUrlPaymentTypeEnum
from ..models import PaymentUrlCreatePaymentProviderEnum


@deprecated
@same_doc_as(GetPaymentCustomization)
def get_payment_customization(
    payment_provider: Union[str, GetPaymentCustomizationPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentCustomization.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetPaymentCustomization)
async def get_payment_customization_async(
    payment_provider: Union[str, GetPaymentCustomizationPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentCustomization.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentPublicConfig)
def get_payment_public_config(
    payment_provider: Union[str, GetPaymentPublicConfigPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentPublicConfig.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentPublicConfig)
async def get_payment_public_config_async(
    payment_provider: Union[str, GetPaymentPublicConfigPaymentProviderEnum],
    region: str,
    sandbox: Optional[bool] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentPublicConfig.create(
        payment_provider=payment_provider,
        region=region,
        sandbox=sandbox,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetPaymentTaxValue)
def get_payment_tax_value(
    payment_order_no: str,
    payment_provider: Union[str, GetPaymentTaxValuePaymentProviderEnum],
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentTaxValue.create(
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetPaymentTaxValue)
async def get_payment_tax_value_async(
    payment_order_no: str,
    payment_provider: Union[str, GetPaymentTaxValuePaymentProviderEnum],
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetPaymentTaxValue.create(
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(Pay)
def pay(
    payment_order_no: str,
    body: Optional[PaymentToken] = None,
    payment_provider: Optional[Union[str, PayPaymentProviderEnum]] = None,
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Pay.create(
        payment_order_no=payment_order_no,
        body=body,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Pay)
async def pay_async(
    payment_order_no: str,
    body: Optional[PaymentToken] = None,
    payment_provider: Optional[Union[str, PayPaymentProviderEnum]] = None,
    zip_code: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Pay.create(
        payment_order_no=payment_order_no,
        body=body,
        payment_provider=payment_provider,
        zip_code=zip_code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicCheckPaymentOrderPaidStatus)
def public_check_payment_order_paid_status(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicCheckPaymentOrderPaidStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicCheckPaymentOrderPaidStatus)
async def public_check_payment_order_paid_status_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicCheckPaymentOrderPaidStatus.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPaymentMethods)
def public_get_payment_methods(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentMethods.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPaymentMethods)
async def public_get_payment_methods_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentMethods.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetPaymentUrl)
def public_get_payment_url(
    body: Optional[PaymentUrlCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentUrl.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetPaymentUrl)
async def public_get_payment_url_async(
    body: Optional[PaymentUrlCreate] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetPaymentUrl.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetQRCode)
def public_get_qr_code(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetQRCode.create(
        code=code,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetQRCode)
async def public_get_qr_code_async(
    code: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetQRCode.create(
        code=code,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetUnpaidPaymentOrder)
def public_get_unpaid_payment_order(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetUnpaidPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetUnpaidPaymentOrder)
async def public_get_unpaid_payment_order_async(
    payment_order_no: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicGetUnpaidPaymentOrder.create(
        payment_order_no=payment_order_no,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicNormalizePaymentReturnUrl)
def public_normalize_payment_return_url(
    order_no: str,
    payment_order_no: str,
    payment_provider: Union[str, PublicNormalizePaymentReturnUrlPaymentProviderEnum],
    return_url: str,
    foreinginvoice: Optional[str] = None,
    invoice_id: Optional[str] = None,
    payer_id: Optional[str] = None,
    payload: Optional[str] = None,
    redirect_result: Optional[str] = None,
    result_code: Optional[str] = None,
    session_id: Optional[str] = None,
    status: Optional[str] = None,
    token: Optional[str] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicNormalizePaymentReturnUrl.create(
        order_no=order_no,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        return_url=return_url,
        foreinginvoice=foreinginvoice,
        invoice_id=invoice_id,
        payer_id=payer_id,
        payload=payload,
        redirect_result=redirect_result,
        result_code=result_code,
        session_id=session_id,
        status=status,
        token=token,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicNormalizePaymentReturnUrl)
async def public_normalize_payment_return_url_async(
    order_no: str,
    payment_order_no: str,
    payment_provider: Union[str, PublicNormalizePaymentReturnUrlPaymentProviderEnum],
    return_url: str,
    foreinginvoice: Optional[str] = None,
    invoice_id: Optional[str] = None,
    payer_id: Optional[str] = None,
    payload: Optional[str] = None,
    redirect_result: Optional[str] = None,
    result_code: Optional[str] = None,
    session_id: Optional[str] = None,
    status: Optional[str] = None,
    token: Optional[str] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicNormalizePaymentReturnUrl.create(
        order_no=order_no,
        payment_order_no=payment_order_no,
        payment_provider=payment_provider,
        return_url=return_url,
        foreinginvoice=foreinginvoice,
        invoice_id=invoice_id,
        payer_id=payer_id,
        payload=payload,
        redirect_result=redirect_result,
        result_code=result_code,
        session_id=session_id,
        status=status,
        token=token,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Ds Log Manager Service."""

__version__ = "2.4.2"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from ._all_terminated_servers import batch_download_server_logs
from ._all_terminated_servers import batch_download_server_logs_async
from ._all_terminated_servers import list_all_terminated_servers
from ._all_terminated_servers import list_all_terminated_servers_async

from ._dslogmanager_operations import public_get_messages
from ._dslogmanager_operations import public_get_messages_async

from ._terminated_servers import check_server_logs
from ._terminated_servers import check_server_logs_async
from ._terminated_servers import download_server_logs
from ._terminated_servers import download_server_logs_async
from ._terminated_servers import list_terminated_servers
from ._terminated_servers import list_terminated_servers_async



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsCreateImagePatchRequest
from ..models import ModelsCreateImageRequest
from ..models import ModelsGetImageDetailResponse
from ..models import ModelsGetImageLimitResponse
from ..models import ModelsGetImagePatchDetailResponse
from ..models import ModelsImageRecord
from ..models import ModelsImageRecordUpdate
from ..models import ModelsImportResponse
from ..models import ModelsListImagePatchesResponse
from ..models import ModelsListImageResponse
from ..models import ResponseError

from ..operations.image_config import CreateImage
from ..operations.image_config import CreateImagePatch
from ..operations.image_config import DeleteImage
from ..operations.image_config import DeleteImagePatch
from ..operations.image_config import ExportImages
from ..operations.image_config import GetImageDetail
from ..operations.image_config import GetImageLimit
from ..operations.image_config import GetImagePatchDetail
from ..operations.image_config import GetImagePatches
from ..operations.image_config import ImageDetailClient
from ..operations.image_config import ImageLimitClient
from ..operations.image_config import ImportImages
from ..operations.image_config import ListImages
from ..operations.image_config import ListImagesSortByEnum, ListImagesSortDirectionEnum
from ..operations.image_config import UpdateImage


@same_doc_as(CreateImage)
def create_image(
    body: ModelsCreateImageRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = CreateImage.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateImage)
async def create_image_async(
    body: ModelsCreateImageRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = CreateImage.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateImagePatch)
def create_image_patch(
    body: ModelsCreateImagePatchRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = CreateImagePatch.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateImagePatch)
async def create_image_patch_async(
    body: ModelsCreateImagePatchRequest,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = CreateImagePatch.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteImage)
def delete_image(
    image_uri: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteImage.create(
        image_uri=image_uri,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteImage)
async def delete_image_async(
    image_uri: str,
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteImage.create(
        image_uri=image_uri,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteImagePatch)
def delete_image_patch(
    image_uri: str,
    version: str,
    version_patch: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteImagePatch.create(
        image_uri=image_uri,
        version=version,
        version_patch=version_patch,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteImagePatch)
async def delete_image_patch_async(
    image_uri: str,
    version: str,
    version_patch: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteImagePatch.create(
        image_uri=image_uri,
        version=version,
        version_patch=version_patch,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ExportImages)
def export_images(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ExportImages.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ExportImages)
async def export_images_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ExportImages.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetImageDetail)
def get_image_detail(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImageDetail.create(
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetImageDetail)
async def get_image_detail_async(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImageDetail.create(
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetImageLimit)
def get_image_limit(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImageLimit.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetImageLimit)
async def get_image_limit_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImageLimit.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetImagePatchDetail)
def get_image_patch_detail(
    version: str,
    version_patch: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImagePatchDetail.create(
        version=version,
        version_patch=version_patch,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetImagePatchDetail)
async def get_image_patch_detail_async(
    version: str,
    version_patch: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImagePatchDetail.create(
        version=version,
        version_patch=version_patch,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetImagePatches)
def get_image_patches(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImagePatches.create(
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetImagePatches)
async def get_image_patches_async(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetImagePatches.create(
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ImageDetailClient)
def image_detail_client(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImageDetailClient.create(
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ImageDetailClient)
async def image_detail_client_async(
    version: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImageDetailClient.create(
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ImageLimitClient)
def image_limit_client(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImageLimitClient.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ImageLimitClient)
async def image_limit_client_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ImageLimitClient.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ImportImages)
def import_images(
    file: Any, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = ImportImages.create(
        file=file,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ImportImages)
async def import_images_async(
    file: Any, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = ImportImages.create(
        file=file,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListImages)
def list_images(
    count: int,
    offset: int,
    q: Optional[str] = None,
    sort_by: Optional[Union[str, ListImagesSortByEnum]] = None,
    sort_direction: Optional[Union[str, ListImagesSortDirectionEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListImages.create(
        count=count,
        offset=offset,
        q=q,
        sort_by=sort_by,
        sort_direction=sort_direction,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListImages)
async def list_images_async(
    count: int,
    offset: int,
    q: Optional[str] = None,
    sort_by: Optional[Union[str, ListImagesSortByEnum]] = None,
    sort_direction: Optional[Union[str, ListImagesSortDirectionEnum]] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListImages.create(
        count=count,
        offset=offset,
        q=q,
        sort_by=sort_by,
        sort_direction=sort_direction,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateImage)
def update_image(
    body: ModelsImageRecordUpdate,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = UpdateImage.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateImage)
async def update_image_async(
    body: ModelsImageRecordUpdate,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    request = UpdateImage.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Cloud Legal Service (1.22.6)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import RetrievePolicyTypeResponse


class RetrieveAllPolicyTypes(Operation):
    """Retrieve All Policy Type (retrieveAllPolicyTypes)

    Retrieve all supported policy types.
    Other detail info:

      * Required permission : resource="ADMIN:NAMESPACE:*:LEGAL", action=2 (READ)

    Required Permission(s):
        - ADMIN:NAMESPACE:*:LEGAL [READ]

    Properties:
        url: /agreement/admin/policy-types

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH] or [BEARER_AUTH]

        offset: (offset) OPTIONAL int in query

        limit: (limit) REQUIRED int in query

    Responses:
        200: OK - List[RetrievePolicyTypeResponse] (successful operation)
    """

    # region fields

    _url: str = "/agreement/admin/policy-types"
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    offset: int  # OPTIONAL in [query]
    limit: int  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "query": self.get_query_params(),
        }

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "offset"):
            result["offset"] = self.offset
        if hasattr(self, "limit"):
            result["limit"] = self.limit
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_offset(self, value: int) -> RetrieveAllPolicyTypes:
        self.offset = value
        return self

    def with_limit(self, value: int) -> RetrieveAllPolicyTypes:
        self.limit = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "offset") and self.offset:
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        if hasattr(self, "limit") and self.limit:
            result["limit"] = int(self.limit)
        elif include_empty:
            result["limit"] = 0
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, List[RetrievePolicyTypeResponse]], Union[None, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - List[RetrievePolicyTypeResponse] (successful operation)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return [
                RetrievePolicyTypeResponse.create_from_dict(i) for i in content
            ], None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        limit: int,
        offset: Optional[int] = None,
    ) -> RetrieveAllPolicyTypes:
        instance = cls()
        instance.limit = limit
        if offset is not None:
            instance.offset = offset
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> RetrieveAllPolicyTypes:
        instance = cls()
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        if "limit" in dict_ and dict_["limit"] is not None:
            instance.limit = int(dict_["limit"])
        elif include_empty:
            instance.limit = 0
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "offset": "offset",
            "limit": "limit",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "offset": False,
            "limit": True,
        }

    # endregion static methods
